@@ -31,9 +31,7 @@ namespace QuantLib {
                            CPI::InterpolationType interpolationType) {
 
         switch (interpolationType) {
-          case AsIndex: {
-              return index->fixing(date - observationLag);
-          }
+          case AsIndex:
           case Flat: {
               auto fixingPeriod = inflationPeriod(date - observationLag, index->frequency());
               return index->fixing(fixingPeriod.first);
@@ -42,16 +40,17 @@ namespace QuantLib {
               auto fixingPeriod = inflationPeriod(date - observationLag, index->frequency());
               auto interpolationPeriod = inflationPeriod(date, index->frequency());
 
+              auto I0 = index->fixing(fixingPeriod.first);
+
               if (date == interpolationPeriod.first) {
                   // special case; no interpolation.  This avoids asking for
                   // the fixing at the end of the period, which might need a
                   // forecast curve to be set.
-                  return index->fixing(fixingPeriod.first);
+                  return I0;
               }
 
               static const auto oneDay = Period(1, Days);
 
-              auto I0 = index->fixing(fixingPeriod.first);
               auto I1 = index->fixing(fixingPeriod.second + oneDay);
 
               return I0 + (I1 - I0) * (date - interpolationPeriod.first) /
@@ -63,28 +62,60 @@ namespace QuantLib {
     }
 
 
-    InflationIndex::InflationIndex(std::string familyName,
-                                   Region region,
-                                   bool revised,
-                                   bool interpolated,
-                                   Frequency frequency,
-                                   const Period& availabilityLag,
-                                   Currency currency)
-    : InflationIndex(std::move(familyName),
-                     std::move(region),
-                     revised,
-                     frequency,
-                     availabilityLag,
-                     std::move(currency)) {
-        QL_DEPRECATED_DISABLE_WARNING
-        interpolated_ = interpolated;
-        QL_DEPRECATED_ENABLE_WARNING
+    Real CPI::laggedYoYRate(const ext::shared_ptr<YoYInflationIndex>& index,
+                            const Date& date,
+                            const Period& observationLag,
+                            CPI::InterpolationType interpolationType) {
+
+        switch (interpolationType) {
+          case AsIndex: {
+              return index->fixing(date - observationLag);
+          }
+          case Flat: {
+              auto fixingPeriod = inflationPeriod(date - observationLag, index->frequency());
+              return index->fixing(fixingPeriod.first);
+          }
+          case Linear: {
+              if (index->ratio() && !index->needsForecast(date)) {
+                  // in the case of a ratio, the convention seems to be to interpolate
+                  // the underlying index fixings first, then take the ratio.  This is
+                  // not the same as taking the ratios and then interpolate, which is
+                  // equivalent to what the else clause does.
+                  // However, we can only do this if the fixings we need are in the past,
+                  // because forecasts need to be done through the YoY forecast curve,
+                  // and not the underlying index.
+
+                  auto underlying = index->underlyingIndex();
+                  Rate Z1 = CPI::laggedFixing(underlying, date, observationLag, interpolationType);
+                  Rate Z0 = CPI::laggedFixing(underlying, date - 1*Years, observationLag, interpolationType);
+
+                  return Z1/Z0 - 1.0;
+
+              } else {
+                  static const auto oneDay = Period(1, Days);
+
+                  auto fixingPeriod = inflationPeriod(date - observationLag, index->frequency());
+                  auto interpolationPeriod = inflationPeriod(date, index->frequency());
+
+                  auto Y0 = index->fixing(fixingPeriod.first);
+
+                  if (date == interpolationPeriod.first) {
+                      // special case; no interpolation anyway.
+                      return Y0;
+                  }
+
+                  auto Y1 = index->fixing(fixingPeriod.second + oneDay);
+
+                  return Y0 + (Y1 - Y0) * (date - interpolationPeriod.first) /
+                      (Real)((interpolationPeriod.second + oneDay) - interpolationPeriod.first);
+              }
+          }
+          default:
+            QL_FAIL("unknown CPI interpolation type: " << int(interpolationType));
+        }
     }
 
-    /* gcc complains in the constructor about the deprecated call of initializing
-     * `bool InflationIndex::interpolated_ = false;` that's why we need to disable the warning here.
-     */
-    QL_DEPRECATED_DISABLE_WARNING
+
     InflationIndex::InflationIndex(std::string familyName,
                                    Region region,
                                    bool revised,
@@ -97,13 +128,6 @@ namespace QuantLib {
         registerWith(Settings::instance().evaluationDate());
         registerWith(IndexManager::instance().notifier(InflationIndex::name()));
     }
-    QL_DEPRECATED_ENABLE_WARNING
-
-    bool InflationIndex::interpolated() const {
-        QL_DEPRECATED_DISABLE_WARNING
-        return interpolated_;
-        QL_DEPRECATED_ENABLE_WARNING
-    }
 
     Calendar InflationIndex::fixingCalendar() const {
         static NullCalendar c;
@@ -127,21 +151,6 @@ namespace QuantLib {
                           rates.begin(), forceOverwrite);
     }
 
-    ZeroInflationIndex::ZeroInflationIndex(const std::string& familyName,
-                                           const Region& region,
-                                           bool revised,
-                                           bool interpolated,
-                                           Frequency frequency,
-                                           const Period& availabilityLag,
-                                           const Currency& currency,
-                                           Handle<ZeroInflationTermStructure> zeroInflation)
-    : ZeroInflationIndex(
-          familyName, region, revised, frequency, availabilityLag, currency, std::move(zeroInflation)) {
-        QL_DEPRECATED_DISABLE_WARNING
-        interpolated_ = interpolated;
-        QL_DEPRECATED_ENABLE_WARNING
-    }
-
     ZeroInflationIndex::ZeroInflationIndex(const std::string& familyName,
                                            const Region& region,
                                            bool revised,
@@ -157,76 +166,51 @@ namespace QuantLib {
     Real ZeroInflationIndex::fixing(const Date& fixingDate,
                                     bool /*forecastTodaysFixing*/) const {
         if (!needsForecast(fixingDate)) {
-            std::pair<Date,Date> p = inflationPeriod(fixingDate, frequency_);
-            const TimeSeries<Real>& ts = timeSeries();
-
-            Real I1 = ts[p.first];
+            const Real I1 = pastFixing(fixingDate);
             QL_REQUIRE(I1 != Null<Real>(),
-                       "Missing " << name() << " fixing for " << p.first);
+                       "Missing " << name() << " fixing for "
+                       << inflationPeriod(fixingDate, frequency_).first);
 
-            QL_DEPRECATED_DISABLE_WARNING
-            if (interpolated_ && fixingDate > p.first) {
-            QL_DEPRECATED_ENABLE_WARNING
-
-                Real I2 = ts[p.second+1];
-                QL_REQUIRE(I2 != Null<Real>(),
-                           "Missing " << name() << " fixing for " << p.second+1);
-
-                // Use non-lagged period for interpolation
-                Date observationDate = fixingDate + zeroInflation_->observationLag();
-                std::pair<Date, Date> p2 = inflationPeriod(observationDate, frequency_);
-                Real daysInPeriod = (p2.second + 1) - p2.first;
-                Real interpolationCoefficient = (observationDate - p2.first) / daysInPeriod;
-
-                return I1 + (I2 - I1) * interpolationCoefficient;
-            } else {
-                // we don't need the next fixing
-                return I1;
-            }
+            return I1;
         } else {
             return forecastFixing(fixingDate);
         }
     }
 
+    Real ZeroInflationIndex::pastFixing(const Date& fixingDate) const {
+        const auto p = inflationPeriod(fixingDate, frequency_);
+        const auto& ts = timeSeries();
+        return ts[p.first];
+    }
+
+    Date ZeroInflationIndex::lastFixingDate() const {
+        const auto& fixings = timeSeries();
+        QL_REQUIRE(!fixings.empty(), "no fixings stored for " << name());
+        // attribute fixing to first day of the underlying period
+        return inflationPeriod(fixings.lastDate(), frequency_).first;
+    }
 
     bool ZeroInflationIndex::needsForecast(const Date& fixingDate) const {
 
-        // Stored fixings are always non-interpolated.
-        // If an interpolated fixing is required then
-        // the availability lag + one inflation period
-        // must have passed to use historical fixings
-        // (because you need the next one to interpolate).
-        // The interpolation is calculated (linearly) on demand.
-
         Date today = Settings::instance().evaluationDate();
-        Date todayMinusLag = today - availabilityLag_;
 
-        Date historicalFixingKnown =
-            inflationPeriod(todayMinusLag, frequency_).first-1;
-        Date latestNeededDate = fixingDate;
+        auto latestPossibleHistoricalFixingPeriod =
+            inflationPeriod(today - availabilityLag_, frequency_);
 
-        QL_DEPRECATED_DISABLE_WARNING
-        if (interpolated_) { // might need the next one too
-            std::pair<Date,Date> p = inflationPeriod(fixingDate, frequency_);
-            if (fixingDate > p.first)
-                latestNeededDate += Period(frequency_);
-        }
-        QL_DEPRECATED_ENABLE_WARNING
+        // Zero-index fixings are always non-interpolated.
+        auto fixingPeriod = inflationPeriod(fixingDate, frequency_);
+        Date latestNeededDate = fixingPeriod.first;
 
-        if (latestNeededDate <= historicalFixingKnown) {
+        if (latestNeededDate < latestPossibleHistoricalFixingPeriod.first) {
             // the fixing date is well before the availability lag, so
-            // we know that fixings were provided.
+            // we know that fixings must be provided.
             return false;
-        } else if (latestNeededDate > today) {
-            // the fixing can't be available, no matter what's in the
-            // time series
+        } else if (latestNeededDate > latestPossibleHistoricalFixingPeriod.second) {
+            // the fixing can't be available yet
             return true;
         } else {
-            // we're not sure, but the fixing might be there so we
-            // check.  Todo: check which fixings are not possible, to
-            // avoid using fixings in the future
-            Date first = Date(1, latestNeededDate.month(), latestNeededDate.year());
-            Real f = timeSeries()[first];
+            // we're not sure, but the fixing might be there so we check.
+            Real f = timeSeries()[latestNeededDate];
             return (f == Null<Real>());
         }
     }
@@ -239,48 +223,20 @@ namespace QuantLib {
                    name() << " index fixing at base date " << baseDate << " is not available");
         Real baseFixing = fixing(baseDate);
 
-        std::pair<Date, Date> p = inflationPeriod(fixingDate, frequency_);
+        std::pair<Date, Date> fixingPeriod = inflationPeriod(fixingDate, frequency_);
 
-        QL_DEPRECATED_DISABLE_WARNING
-
-        Date firstDateInPeriod = p.first;
+        Date firstDateInPeriod = fixingPeriod.first;
         Rate Z1 = zeroInflation_->zeroRate(firstDateInPeriod, Period(0,Days), false);
-        Time t1 = inflationYearFraction(frequency_, interpolated_, zeroInflation_->dayCounter(),
+        Time t1 = inflationYearFraction(frequency_, false, zeroInflation_->dayCounter(),
                                         baseDate, firstDateInPeriod);
-        Real I1 = baseFixing * std::pow(1.0 + Z1, t1);
-
-        if (interpolated_ && fixingDate > firstDateInPeriod) {
-            Date firstDateInNextPeriod = p.second + 1;
-            Rate Z2 = zeroInflation_->zeroRate(firstDateInNextPeriod, Period(0,Days), false);
-            Time t2 = inflationYearFraction(frequency_, interpolated_, zeroInflation_->dayCounter(),
-                                            baseDate, firstDateInNextPeriod);
-            Real I2 = baseFixing * std::pow(1.0 + Z2, t2);
-
-            // // Use non-lagged period for interpolation
-            Date observationDate = fixingDate + zeroInflation_->observationLag();
-            std::pair<Date, Date> p2 = inflationPeriod(observationDate, frequency_);
-            Real daysInPeriod = (p2.second + 1) - p2.first;
-            Real interpolationCoefficient = (observationDate - p2.first) / daysInPeriod;
-
-            return I1 + (I2 - I1) * interpolationCoefficient;
-        } else {
-            return I1;
-        }
-
-        QL_DEPRECATED_ENABLE_WARNING
+        return baseFixing * std::pow(1.0 + Z1, t1);
     }
 
 
     ext::shared_ptr<ZeroInflationIndex> ZeroInflationIndex::clone(
                           const Handle<ZeroInflationTermStructure>& h) const {
-        /* using the new constructor and set interpolated to avoid the deprecated warning and
-         * error...  */
-        auto clonedIndex = ext::make_shared<ZeroInflationIndex>(
+        return ext::make_shared<ZeroInflationIndex>(
             familyName_, region_, revised_, frequency_, availabilityLag_, currency_, h);
-        QL_DEPRECATED_DISABLE_WARNING
-        clonedIndex->interpolated_ = interpolated_;
-        QL_DEPRECATED_ENABLE_WARNING
-        return clonedIndex;
     }
 
 
@@ -296,8 +252,6 @@ namespace QuantLib {
         registerWith(yoyInflation_);
     }
 
-    QL_DEPRECATED_DISABLE_WARNING
-
     YoYInflationIndex::YoYInflationIndex(const std::string& familyName,
                                          const Region& region,
                                          bool revised,
@@ -306,49 +260,64 @@ namespace QuantLib {
                                          const Period& availabilityLag,
                                          const Currency& currency,
                                          Handle<YoYInflationTermStructure> yoyInflation)
-    : YoYInflationIndex(familyName, region, revised, interpolated, false,
-                        frequency, availabilityLag, currency, std::move(yoyInflation)) {}
-
-    QL_DEPRECATED_ENABLE_WARNING
-
-    YoYInflationIndex::YoYInflationIndex(const std::string& familyName,
-                                         const Region& region,
-                                         bool revised,
-                                         bool interpolated,
-                                         bool ratio,
-                                         Frequency frequency,
-                                         const Period& availabilityLag,
-                                         const Currency& currency,
-                                         Handle<YoYInflationTermStructure> yoyInflation)
     : InflationIndex(familyName, region, revised, frequency, availabilityLag, currency),
-      interpolated_(interpolated), ratio_(ratio), yoyInflation_(std::move(yoyInflation)) {
-        if (ratio)
-            underlyingIndex_ = ext::make_shared<ZeroInflationIndex>(familyName, region, revised,
-                                                                    frequency, availabilityLag, currency);
+      interpolated_(interpolated), ratio_(false), yoyInflation_(std::move(yoyInflation)) {
         registerWith(yoyInflation_);
     }
 
 
     Rate YoYInflationIndex::fixing(const Date& fixingDate,
                                    bool /*forecastTodaysFixing*/) const {
+        if (needsForecast(fixingDate)) {
+            return forecastFixing(fixingDate);
+        } else {
+            return pastFixing(fixingDate);
+        }
+    }
 
+    Date YoYInflationIndex::lastFixingDate() const {
+        if (ratio()) {
+            return underlyingIndex_->lastFixingDate();
+        } else {
+            const auto& fixings = timeSeries();
+            QL_REQUIRE(!fixings.empty(), "no fixings stored for " << name());
+            // attribute fixing to first day of the underlying period
+            return inflationPeriod(fixings.lastDate(), frequency_).first;
+        }
+    }
+
+    bool YoYInflationIndex::needsForecast(const Date& fixingDate) const {
         Date today = Settings::instance().evaluationDate();
-        Date todayMinusLag = today - availabilityLag_;
-        std::pair<Date,Date> lim = inflationPeriod(todayMinusLag, frequency_);
-        Date lastFix = lim.first-1;
 
-        Date flatMustForecastOn = lastFix+1;
-        Date interpMustForecastOn = lastFix+1 - Period(frequency_);
+        auto fixingPeriod = inflationPeriod(fixingDate, frequency_);
+        Date latestNeededDate;
+        if (!interpolated() || fixingDate == fixingPeriod.first)
+            latestNeededDate = fixingPeriod.first;
+        else
+            latestNeededDate = fixingPeriod.second + 1;
 
-        if (interpolated() && fixingDate >= interpMustForecastOn) {
-            return forecastFixing(fixingDate);
+        if (ratio()) {
+            return underlyingIndex_->needsForecast(latestNeededDate);
+        } else {
+            auto latestPossibleHistoricalFixingPeriod =
+                inflationPeriod(today - availabilityLag_, frequency_);
+
+            if (latestNeededDate < latestPossibleHistoricalFixingPeriod.first) {
+                // the fixing date is well before the availability lag, so
+                // we know that fixings must be provided.
+                return false;
+            } else if (latestNeededDate > latestPossibleHistoricalFixingPeriod.second) {
+                // the fixing can't be available yet
+                return true;
+            } else {
+                // we're not sure, but the fixing might be there so we check.
+                Real f = timeSeries()[latestNeededDate];
+                return (f == Null<Real>());
+            }
         }
+    }
 
-        if (!interpolated() && fixingDate >= flatMustForecastOn) {
-            return forecastFixing(fixingDate);
-        }
-
-        const TimeSeries<Real>& ts = timeSeries();
+    Real YoYInflationIndex::pastFixing(const Date& fixingDate) const {
         if (ratio()) {
 
             auto interpolationType = interpolated() ? CPI::Linear : CPI::Flat;
@@ -360,37 +329,30 @@ namespace QuantLib {
 
         } else {  // NOT ratio
 
-            if (interpolated()) { // NOT ratio, IS interpolated
+            const auto& ts = timeSeries();
+            auto [periodStart, periodEnd] = inflationPeriod(fixingDate, frequency_);
 
-                std::pair<Date,Date> lim = inflationPeriod(fixingDate, frequency_);
-                Real dp = lim.second + 1 - lim.first;
-                Real dl = fixingDate - lim.first;
-                Rate limFirstFix = ts[lim.first];
-                QL_REQUIRE(limFirstFix != Null<Rate>(),
-                            "Missing " << name() << " fixing for "
-                            << lim.first );
-                Rate limSecondFix = ts[lim.second+1];
-                QL_REQUIRE(limSecondFix != Null<Rate>(),
-                            "Missing " << name() << " fixing for "
-                            << lim.second+1 );
-                Real linearNow = limFirstFix + (limSecondFix-limFirstFix)*dl/dp;
+            Rate YY0 = ts[periodStart];
+            QL_REQUIRE(YY0 != Null<Rate>(),
+                       "Missing " << name() << " fixing for " << periodStart);
 
-                return linearNow;
+            if (!interpolated() || /* degenerate case */ fixingDate == periodStart) {
 
-            } else { // NOT ratio, NOT interpolated
-                     // so just flat
+                return YY0;
 
-                std::pair<Date,Date> lim = inflationPeriod(fixingDate, frequency_);
-                Rate pastFixing = ts[lim.first];
-                QL_REQUIRE(pastFixing != Null<Rate>(),
-                           "Missing " << name() << " fixing for " << lim.first);
-                return pastFixing;
+            } else {
+
+                Real dp = periodEnd + 1 - periodStart;
+                Real dl = fixingDate - periodStart;
+                Rate YY1 = ts[periodEnd+1];
+                QL_REQUIRE(YY1 != Null<Rate>(),
+                           "Missing " << name() << " fixing for " << periodEnd+1);
+                return YY0 + (YY1 - YY0) * dl / dp;
 
             }
         }
     }
 
-
     Real YoYInflationIndex::forecastFixing(const Date& fixingDate) const {
 
         Date d;
@@ -399,8 +361,8 @@ namespace QuantLib {
         } else {
             // if the value is not interpolated use the starting value
             // by internal convention this will be consistent
-            std::pair<Date,Date> lim = inflationPeriod(fixingDate, frequency_);
-            d = lim.first;
+            std::pair<Date,Date> fixingPeriod = inflationPeriod(fixingDate, frequency_);
+            d = fixingPeriod.first;
         }
         return yoyInflation_->yoyRate(d,0*Days);
     }
@@ -418,15 +380,31 @@ namespace QuantLib {
 
 
     CPI::InterpolationType
-    detail::CPI::effectiveInterpolationType(const ext::shared_ptr<ZeroInflationIndex>& index,
-                                            const QuantLib::CPI::InterpolationType& type) {
+    detail::CPI::effectiveInterpolationType(const QuantLib::CPI::InterpolationType& type) {
         if (type == QuantLib::CPI::AsIndex) {
-            QL_DEPRECATED_DISABLE_WARNING
-            return index->interpolated() ? QuantLib::CPI::Linear : QuantLib::CPI::Flat;
-            QL_DEPRECATED_ENABLE_WARNING
+            return QuantLib::CPI::Flat;
         } else {
             return type;
         }
     }
 
+    CPI::InterpolationType
+    detail::CPI::effectiveInterpolationType(const QuantLib::CPI::InterpolationType& type,
+                                            const ext::shared_ptr<YoYInflationIndex>& index) {
+        if (type == QuantLib::CPI::AsIndex) {
+            return index->interpolated() ? QuantLib::CPI::Linear : QuantLib::CPI::Flat;
+        } else {
+            return type;
+        }
+    }
+
+    bool detail::CPI::isInterpolated(const QuantLib::CPI::InterpolationType& type) {
+        return detail::CPI::effectiveInterpolationType(type) == QuantLib::CPI::Linear;
+    }
+
+    bool detail::CPI::isInterpolated(const QuantLib::CPI::InterpolationType& type,
+                                     const ext::shared_ptr<YoYInflationIndex>& index) {
+        return detail::CPI::effectiveInterpolationType(type, index) == QuantLib::CPI::Linear;
+    }
+
 }
