@@ -19,6 +19,7 @@
 
 #include <ql/termstructures/volatility/smilesectionutils.hpp>
 #include <ql/math/comparison.hpp>
+#include <algorithm>
 
 namespace QuantLib {
 
@@ -81,8 +82,8 @@ namespace QuantLib {
         }
 
         bool minStrikeAdded = false, maxStrikeAdded = false;
-        for (double& i : tmp) {
-            Real k = section.volatilityType() == Normal ? f_ + i : i * (f_ + shift) - shift;
+        for (Real& i : tmp) {
+            Real k = section.volatilityType() == Normal ? Real(f_ + i) : Real(i * (f_ + shift) - shift);
             if ((section.volatilityType() == ShiftedLognormal && i <= QL_EPSILON) ||
                 (k >= section.minStrike() && k <= section.maxStrike())) {
                 if (!minStrikeAdded || !close(k, section.minStrike())) {
@@ -96,15 +97,15 @@ namespace QuantLib {
                      // in order to not loose too much information
                 if (k < section.minStrike() && !minStrikeAdded) {
                     m_.push_back(section.volatilityType() == Normal
-                                     ? section.minStrike() - f_
-                                     : (section.minStrike() + shift) / f_);
+                                     ? Real(section.minStrike() - f_)
+                                     : Real((section.minStrike() + shift) / f_));
                     k_.push_back(section.minStrike());
                     minStrikeAdded = true;
                 }
                 if (k > section.maxStrike() && !maxStrikeAdded) {
                     m_.push_back(section.volatilityType() == Normal
-                                     ? section.maxStrike() - f_
-                                     : (section.maxStrike() + shift) / f_);
+                                     ? Real(section.maxStrike() - f_)
+                                     : Real((section.maxStrike() + shift) / f_));
                     k_.push_back(section.maxStrike());
                     maxStrikeAdded = true;
                 }
@@ -138,7 +139,7 @@ namespace QuantLib {
             centralIndex++;
 
         QL_REQUIRE(centralIndex < k_.size(),
-                   "central index is at right boundary")
+                   "central index is at right boundary");
 
         leftIndex_ = centralIndex;
         rightIndex_ = centralIndex;
