@@ -18,14 +18,15 @@
  FOR A PARTICULAR PURPOSE.  See the license for more details.
 */
 
-#include <ql/termstructures/yield/fittedbonddiscountcurve.hpp>
-#include <ql/pricingengines/bond/bondfunctions.hpp>
-#include <ql/math/optimization/simplex.hpp>
-#include <ql/math/optimization/costfunction.hpp>
-#include <ql/math/optimization/constraint.hpp>
 #include <ql/cashflows/cashflows.hpp>
-#include <ql/utilities/dataformatters.hpp>
+#include <ql/math/optimization/constraint.hpp>
+#include <ql/math/optimization/costfunction.hpp>
+#include <ql/math/optimization/simplex.hpp>
+#include <ql/pricingengines/bond/bondfunctions.hpp>
+#include <ql/termstructures/yield/fittedbonddiscountcurve.hpp>
 #include <ql/time/daycounters/simpledaycounter.hpp>
+#include <ql/utilities/dataformatters.hpp>
+#include <utility>
 
 using std::vector;
 
@@ -37,63 +38,61 @@ namespace QuantLib {
       public:
         explicit FittingCost(
                        FittedBondDiscountCurve::FittingMethod* fittingMethod);
-        Real value(const Array& x) const;
-        Disposable<Array> values(const Array& x) const;
+        Real value(const Array& x) const override;
+        Array values(const Array& x) const override;
+
       private:
         FittedBondDiscountCurve::FittingMethod* fittingMethod_;
-        mutable vector<Size> firstCashFlow_;
     };
 
 
-    FittedBondDiscountCurve::FittedBondDiscountCurve (
-                 Natural settlementDays,
-                 const Calendar& calendar,
-                 const vector<ext::shared_ptr<BondHelper> >& bondHelpers,
-                 const DayCounter& dayCounter,
-                 const FittingMethod& fittingMethod,
-                 Real accuracy,
-                 Size maxEvaluations,
-                 const Array& guess,
-                 Real simplexLambda,
-                 Size maxStationaryStateIterations)
-    : YieldTermStructure(settlementDays, calendar, dayCounter),
-      accuracy_(accuracy),
-      maxEvaluations_(maxEvaluations),
-      simplexLambda_(simplexLambda),
-      maxStationaryStateIterations_(maxStationaryStateIterations),
-      guessSolution_(guess),
-      bondHelpers_(bondHelpers),
-      fittingMethod_(fittingMethod) {
+    FittedBondDiscountCurve::FittedBondDiscountCurve(
+        Natural settlementDays,
+        const Calendar& calendar,
+        vector<ext::shared_ptr<BondHelper> > bondHelpers,
+        const DayCounter& dayCounter,
+        const FittingMethod& fittingMethod,
+        Real accuracy,
+        Size maxEvaluations,
+        Array guess,
+        Real simplexLambda,
+        Size maxStationaryStateIterations)
+    : YieldTermStructure(settlementDays, calendar, dayCounter), accuracy_(accuracy),
+      maxEvaluations_(maxEvaluations), simplexLambda_(simplexLambda),
+      maxStationaryStateIterations_(maxStationaryStateIterations), guessSolution_(std::move(guess)),
+      bondHelpers_(std::move(bondHelpers)), fittingMethod_(fittingMethod) {
+        fittingMethod_->curve_ = this;
+        setup();
+    }
+
+
+    FittedBondDiscountCurve::FittedBondDiscountCurve(
+        const Date& referenceDate,
+        vector<ext::shared_ptr<BondHelper> > bondHelpers,
+        const DayCounter& dayCounter,
+        const FittingMethod& fittingMethod,
+        Real accuracy,
+        Size maxEvaluations,
+        Array guess,
+        Real simplexLambda,
+        Size maxStationaryStateIterations)
+    : YieldTermStructure(referenceDate, Calendar(), dayCounter), accuracy_(accuracy),
+      maxEvaluations_(maxEvaluations), simplexLambda_(simplexLambda),
+      maxStationaryStateIterations_(maxStationaryStateIterations), guessSolution_(std::move(guess)),
+      bondHelpers_(std::move(bondHelpers)), fittingMethod_(fittingMethod) {
 
         fittingMethod_->curve_ = this;
         setup();
     }
 
 
-    FittedBondDiscountCurve::FittedBondDiscountCurve (
-                 const Date& referenceDate,
-                 const vector<ext::shared_ptr<BondHelper> >& bondHelpers,
-                 const DayCounter& dayCounter,
-                 const FittingMethod& fittingMethod,
-                 Real accuracy,
-                 Size maxEvaluations,
-                 const Array& guess,
-                 Real simplexLambda,
-                 Size maxStationaryStateIterations)
-    : YieldTermStructure(referenceDate, Calendar(), dayCounter),
-      accuracy_(accuracy),
-      maxEvaluations_(maxEvaluations),
-      simplexLambda_(simplexLambda),
-      maxStationaryStateIterations_(maxStationaryStateIterations),
-      guessSolution_(guess),
-      bondHelpers_(bondHelpers),
-      fittingMethod_(fittingMethod) {
-
-        fittingMethod_->curve_ = this;
-        setup();
+    void FittedBondDiscountCurve::resetGuess(const Array& guess) {
+        QL_REQUIRE(guess.empty() || guess.size() == fittingMethod_->size(), "guess is of wrong size");
+        guessSolution_ = guess;
+        update();
     }
 
-
+    
     void FittedBondDiscountCurve::performCalculations() const {
 
         QL_REQUIRE(!bondHelpers_.empty(), "no bondHelpers given");
@@ -126,12 +125,20 @@ namespace QuantLib {
 
 
     FittedBondDiscountCurve::FittingMethod::FittingMethod(
-                     bool constrainAtZero,
-                     const Array& weights,
-                     ext::shared_ptr<OptimizationMethod> optimizationMethod,
-                     const Array& l2)
-    : constrainAtZero_(constrainAtZero), weights_(weights), l2_(l2),
-      calculateWeights_(weights.empty()), optimizationMethod_(optimizationMethod) {}
+        bool constrainAtZero,
+        const Array& weights,
+        ext::shared_ptr<OptimizationMethod> optimizationMethod,
+        Array l2,
+        const Real minCutoffTime,
+        const Real maxCutoffTime,
+        Constraint constraint)
+    : constrainAtZero_(constrainAtZero), weights_(weights), l2_(std::move(l2)),
+      calculateWeights_(weights.empty()), optimizationMethod_(std::move(optimizationMethod)),
+      constraint_(std::move(constraint)),
+      minCutoffTime_(minCutoffTime), maxCutoffTime_(maxCutoffTime) {
+        if (constraint_.empty())
+            constraint_ = NoConstraint();
+    }
 
     void FittedBondDiscountCurve::FittingMethod::init() {
         // yield conventions
@@ -141,18 +148,9 @@ namespace QuantLib {
 
         Size n = curve_->bondHelpers_.size();
         costFunction_ = ext::make_shared<FittingCost>(this);
-        costFunction_->firstCashFlow_.resize(n);
 
-        for (Size i=0; i<curve_->bondHelpers_.size(); ++i) {
-            ext::shared_ptr<Bond> bond = curve_->bondHelpers_[i]->bond();
-            const Leg& cf = bond->cashflows();
-            Date bondSettlement = bond->settlementDate();
-            for (Size k=0; k<cf.size(); ++k) {
-                if (!cf[k]->hasOccurred(bondSettlement, false)) {
-                    costFunction_->firstCashFlow_[i] = k;
-                    break;
-                }
-            }
+        for (auto& bondHelper : curve_->bondHelpers_) {
+            bondHelper->setTermStructure(curve_);
         }
 
         if (calculateWeights_) {
@@ -163,10 +161,11 @@ namespace QuantLib {
             for (Size i=0; i<curve_->bondHelpers_.size(); ++i) {
                 ext::shared_ptr<Bond> bond = curve_->bondHelpers_[i]->bond();
 
-                Real cleanPrice = curve_->bondHelpers_[i]->quote()->value();
+                Real amount = curve_->bondHelpers_[i]->quote()->value();
+                Bond::Price price(amount, curve_->bondHelpers_[i]->priceType());
 
                 Date bondSettlement = bond->settlementDate();
-                Rate ytm = BondFunctions::yield(*bond, cleanPrice,
+                Rate ytm = BondFunctions::yield(*bond, price,
                                                 yieldDC, yieldComp, yieldFreq,
                                                 bondSettlement);
 
@@ -186,17 +185,19 @@ namespace QuantLib {
         if (!l2_.empty()) {
             QL_REQUIRE(l2_.size() == size(),
                        "Given penalty factors do not cover all parameters");
+
+            QL_REQUIRE(!curve_->guessSolution_.empty(), "L2 penalty requires a guess");
         }
     }
 
     void FittedBondDiscountCurve::FittingMethod::calculate() {
 
         FittingCost& costFunction = *costFunction_;
-        Constraint constraint = NoConstraint();
 
         // start with the guess solution, if it exists
         Array x(size(), 0.0);
         if (!curve_->guessSolution_.empty()) {
+            QL_REQUIRE(curve_->guessSolution_.size() == size(), "wrong size for guess");
             x = curve_->guessSolution_;
         }
 
@@ -214,6 +215,7 @@ namespace QuantLib {
 
             numberOfIterations_ = 0;
             costValue_ = costFunction.value(solution_);
+            errorCode_ = EndCriteria::None;
 
             return;
         }
@@ -223,7 +225,7 @@ namespace QuantLib {
         if(!optimization){
             optimization = ext::make_shared<Simplex>(curve_->simplexLambda_);
         }
-        Problem problem(costFunction, constraint, x);
+        Problem problem(costFunction, constraint_, x);
 
         Real rootEpsilon = curve_->accuracy_;
         Real functionEpsilon =  curve_->accuracy_;
@@ -235,7 +237,7 @@ namespace QuantLib {
                                 functionEpsilon,
                                 gradientNormEpsilon);
 
-        optimization->minimize(problem,endCriteria);
+        errorCode_ = optimization->minimize(problem,endCriteria);
         solution_ = problem.currentValue();
 
         numberOfIterations_ = problem.functionEvaluation();
@@ -255,46 +257,24 @@ namespace QuantLib {
                                                        const Array& x) const {
         Real squaredError = 0.0;
         Array vals = values(x);
-        for (Size i = 0; i<vals.size(); ++i) {
-            squaredError += vals[i];
+        for (Real val : vals) {
+            squaredError += val;
         }
         return squaredError;
     }
 
-    Disposable<Array>
-    FittedBondDiscountCurve::FittingMethod::FittingCost::values(
-                                                       const Array &x) const {
-        Date refDate  = fittingMethod_->curve_->referenceDate();
-        const DayCounter& dc = fittingMethod_->curve_->dayCounter();
+    Array FittedBondDiscountCurve::FittingMethod::FittingCost::values(const Array &x) const {
         Size n = fittingMethod_->curve_->bondHelpers_.size();
         Size N = fittingMethod_->l2_.size();
 
+        // set solution so that fittingMethod_->curve_ represents the current trial
+        // the final solution will be set in FittingMethod::calculate() later on
+        fittingMethod_->solution_ = x;
+
         Array values(n + N);
         for (Size i=0; i<n; ++i) {
-            ext::shared_ptr<BondHelper> helper =
-                fittingMethod_->curve_->bondHelpers_[i];
-
-            ext::shared_ptr<Bond> bond = helper->bond();
-            Date bondSettlement = bond->settlementDate();
-
-            // CleanPrice_i = sum( cf_k * d(t_k) ) - accruedAmount
-            Real modelPrice = 0.0;
-            const Leg& cf = bond->cashflows();
-            for (Size k=firstCashFlow_[i]; k<cf.size(); ++k) {
-                Time tenor = dc.yearFraction(refDate, cf[k]->date());
-                modelPrice += cf[k]->amount() *
-                                    fittingMethod_->discountFunction(x, tenor);
-            }
-            if (helper->priceType() == Bond::Price::Clean)
-                modelPrice -= bond->accruedAmount(bondSettlement);
-
-            // adjust price (NPV) for forward settlement
-            if (bondSettlement != refDate ) {
-                Time tenor = dc.yearFraction(refDate, bondSettlement);
-                modelPrice /= fittingMethod_->discountFunction(x, tenor);
-            }
-            Real marketPrice = helper->quote()->value();
-            Real error = modelPrice - marketPrice;
+            ext::shared_ptr<BondHelper> helper = fittingMethod_->curve_->bondHelpers_[i];
+            Real error = helper->impliedQuote() - helper->quote()->value();
             Real weightedError = fittingMethod_->weights_[i] * error;
             values[i] = weightedError * weightedError;
         }
