@@ -189,12 +189,29 @@ BOOST_AUTO_TEST_CASE(testFlatExtrapolation) {
         // Real modelYield1 = bonds[i]->yield(modelPrices1[i], Actual365Fixed(), Continuous, NoFrequency);
         Real modelYield2 =
             bonds[i]->yield(modelPrices2[i], Actual365Fixed(), Continuous, NoFrequency);
-        // Real curveYield1 = curve1->zeroRate(t, Continuous).rate();
+        Real curveYield1 = curve1->zeroRate(t, Continuous).rate();
         Real curveYield2 = curve2->zeroRate(t, Continuous).rate();
 
+        if (curveYield1 < 1.0) {
+            BOOST_ERROR("Expecting huge yield; the test premise might be outdated");
+        }
         QL_CHECK_CLOSE(modelYield2, curveYield2, 1.0); // 1.0 percent relative tolerance
     }
-    
+
+    // resetting the guess changes the calibration
+
+    curve1->resetGuess({ 0.02, 0.0, 0.0, 0.0 });
+
+    BOOST_CHECK_EQUAL(curve1->fitResults().errorCode(), EndCriteria::StationaryPoint);
+
+    for (Size i = 0; i < helpers.size(); ++i) {
+        Real t = curve1->timeFromReference(helpers[i]->bond()->maturityDate());
+        Real modelYield1 = bonds[i]->yield(modelPrices1[i], Actual365Fixed(), Continuous, NoFrequency);
+        Real curveYield1 = curve1->zeroRate(t, Continuous).rate();
+
+        QL_CHECK_CLOSE(modelYield1, curveYield1, 6); // somewhat better, within a dozen bps
+    }
+
 }
 
 BOOST_AUTO_TEST_CASE(testRequiredGuess) {
@@ -227,6 +244,89 @@ BOOST_AUTO_TEST_CASE(testRequiredGuess) {
                           ExpectedErrorMessage("L2 penalty requires a guess"));
 }
 
+BOOST_AUTO_TEST_CASE(testGuessSize) {
+
+    BOOST_TEST_MESSAGE("Testing that fitted bond curves check the guess size when given...");
+
+    Date today = Settings::instance().evaluationDate();
+    auto bond1 = ext::make_shared<ZeroCouponBond>(3, TARGET(), 100.0, today + Period(1, Years));
+    auto bond2 = ext::make_shared<ZeroCouponBond>(3, TARGET(), 100.0, today + Period(2, Years));
+    auto bond3 = ext::make_shared<ZeroCouponBond>(3, TARGET(), 100.0, today + Period(5, Years));
+    auto bond4 = ext::make_shared<ZeroCouponBond>(3, TARGET(), 100.0, today + Period(10, Years));
+
+    std::vector<ext::shared_ptr<BondHelper> > helpers(4);
+    helpers[0] = ext::make_shared<BondHelper>(makeQuoteHandle(99.0), bond1);
+    helpers[1] = ext::make_shared<BondHelper>(makeQuoteHandle(98.0), bond2);
+    helpers[2] = ext::make_shared<BondHelper>(makeQuoteHandle(95.0), bond3);
+    helpers[3] = ext::make_shared<BondHelper>(makeQuoteHandle(90.0), bond4);
+
+    NelsonSiegelFitting fittingMethod;
+
+    Real accuracy = 1e-10;
+    Size maxIterations = 10000;
+    Array guess = { 0.01, 0.0, 0.0 };  // too few
+    FittedBondDiscountCurve curve(0, TARGET(), helpers, Actual365Fixed(),
+                                  fittingMethod, accuracy, maxIterations, guess);
+
+    BOOST_CHECK_EXCEPTION(curve.discount(3.0), Error,
+                          ExpectedErrorMessage("wrong size for guess"));
+}
+
+
+
+
+BOOST_AUTO_TEST_CASE(testConstraint) {
+
+    BOOST_TEST_MESSAGE("Testing that fitted bond curves respect passed constraint...");
+
+    class FlatZero : public FittedBondDiscountCurve::FittingMethod {
+      public:
+        FlatZero(Constraint constraint = NoConstraint())
+        : FittedBondDiscountCurve::FittingMethod(true,        // constrainAtZero
+                                                 Array(),     // weights
+                                                 ext::shared_ptr<OptimizationMethod>(),
+                                                 Array(),     // l2
+                                                 0.0,         // minCutoffTime
+                                                 QL_MAX_REAL, //maxCutoffTime
+                                                 std::move(constraint)) {}
+
+        std::unique_ptr<FittedBondDiscountCurve::FittingMethod> clone() const override {
+            return std::make_unique<FlatZero>(*this);
+        }
+
+      private:
+        Size size() const override { return 1; }
+
+        DiscountFactor discountFunction(const Array& x, Time t) const override {
+            Real zeroRate = x[0];
+            DiscountFactor d = std::exp(-zeroRate * t);
+            return d;
+        }
+    };
+
+    Date today = Settings::instance().evaluationDate();
+    auto bond1 = ext::make_shared<ZeroCouponBond>(3, TARGET(), 100.0, today + Period(1, Years));
+    auto bond2 = ext::make_shared<ZeroCouponBond>(3, TARGET(), 100.0, today + Period(2, Years));
+
+    std::vector<ext::shared_ptr<BondHelper>> helpers(2);
+    helpers[0] = ext::make_shared<BondHelper>(makeQuoteHandle(101.0), bond1);
+    helpers[1] = ext::make_shared<BondHelper>(makeQuoteHandle(102.0), bond2);
+
+    Real accuracy = 1e-10;      // default value
+    Size maxIterations = 10000; // default value
+    Array guess = {0.01};       // something positive so that initial value is in feasible region
+    
+    FlatZero unconstrainedMethod;
+    FittedBondDiscountCurve unconstrainedCurve(0, TARGET(), helpers, Actual365Fixed(), unconstrainedMethod, 
+                                               accuracy, maxIterations, guess);
+    BOOST_CHECK_LT(unconstrainedCurve.fitResults().solution()[0], 0.0);
+
+    FlatZero positiveMethod{PositiveConstraint()};
+    FittedBondDiscountCurve positiveCurve(0, TARGET(), helpers, Actual365Fixed(), positiveMethod,
+                                          accuracy, maxIterations, guess);
+    BOOST_CHECK_GT(positiveCurve.fitResults().solution()[0], 0.0);
+}
+
 BOOST_AUTO_TEST_SUITE_END()
 
 BOOST_AUTO_TEST_SUITE_END()
